#!/bin/sh
#
# Called from udev
#
# Attempt to mount any added block devices and umount any removed devices

DESKTOP=`awk -v FS=':' -v user="elly" '($1==user) {print $6}' "/etc/passwd"`/Desktop
HOMED=/media
MOUNT="/bin/mount"
PMOUNT="/usr/bin/pmount"
UMOUNT="/bin/umount"
for line in `grep -v ^# /etc/udev/mount.blacklist`
do
	if [ ` expr match "$DEVNAME" "$line" ` -gt 0 ];
	then
		logger "udev/mount.sh" "[$DEVNAME] is blacklisted, ignoring"
		exit 0
	fi
done

automount() {
	# We use ID_FS_LABEL instead of ID_FS_LABEL_ENC
	# ID_FS_LABLE_ENC is the device's name with encoded characters as hex. Unfortunately since we don't know the encoding,
	# we cannot convert these characters to sane UTF16 characters.
	# ID_FS_LABEL is considered the safe label and replaces any encoded characters with underscore(_). 
	# Ubuntu exclusively uses ID_FS_LABEL for the reason of not knowing the encoding.
	if [ "x" = "x$ID_FS_LABEL" ]; then
		name="`basename "$DEVNAME"`"
	else
		name=$ID_FS_LABEL
	fi

	# Check if name is empty somehow
	if [ "x" = "x$name" ]; then
		return
	fi

	# Check if another of the same named drive is plugged in
	while [ -e "/tmp/.automount-$name" ]; do
		# Just prepend an underscore
		name="_$name"
	done


	# Check if this folder already exists at Desktop
	while [ -d "$DESKTOP/$name" ]; do
		# Just prepend an underscore
		name="_$name"
	done

	# Finally try and mkdir and mount
	! test -d "$HOMED/$name" && mkdir -p "$HOMED/$name"

	if ! $MOUNT -t auto -o uid=1000,gid=1000 $DEVNAME "$HOMED/$name"
	then
		#logger "mount.sh/automount" "$MOUNT -t auto $DEVNAME \$HOMED/$name\" failed!"
		rm_dir "$HOMED/$name"
	else
		# symlink it to the Desktop
		ln -s $HOMED/$name $DESKTOP/$name
		logger "mount.sh/automount" "Auto-mount of [$HOMED/$name] successful"
		touch "/tmp/.automount-$name"
	fi

	# Tell the Utility Daemon that a change to the mounted devices has occured. 
	# This will allow applications that depend on this information to request QStorageInfo on mounted devices.
	dbus-send --system --type="method_call" --dest=com.graphicproducts.gputility / local.GPUtilityd.UtilityDaemon.storageDeviceMounted string:"$HOMED/$name"
}

rm_dir() {
	# Tell the Utility Daemon that a mounted device has been removed. 
	# This will allow applications that depend on this information to remove any references to the specified "$HOMED/$name" location
	# We make the dbus call first. This forces the users view away from the mounted drive before we do the remove.
	dbus-send --system --type="method_call" --dest=com.graphicproducts.gputility / local.GPUtilityd.UtilityDaemon.storageDeviceRemoved string:"$HOMED/$name"

	# We do not want to remove populated directories
	removeDirStatus=$(rmdir $1; echo $?)
	if [[ $removeDirStatus -ne 0 ]]; then
		logger "mount.sh/automount" "Not removing non-empty directory [$1]"
	fi

	# We always try to delete the /tmp/.automount-$name
	name="`basename "$1"`"
	if [ -e "/tmp/.automount-$name" ]; then
		rm "/tmp/.automount-$name"
	fi

	# Always delete the symlink too
	if [ -L "$DESKTOP/$name" ]; then
		rm "$DESKTOP/$name";
	fi
}

if [ "$ACTION" = "add" ] && [ -n "$DEVNAME" ] && [ -n "$ID_FS_USAGE" ]; then
	if [ -x "$PMOUNT" ]; then
		$PMOUNT $DEVNAME 2> /dev/null
	elif [ -x $MOUNT ]; then
  		$MOUNT $DEVNAME 2> /dev/null
	fi

	# If the device isn't mounted at this point, it isn't configured in fstab
	# Presently I check for ID_FS_USAGE which has only been present for partitions
	# Another option would be DEVTYPE=partition? Other FS_USAGE are other, crypto
	if [ "$ID_FS_USAGE" = "filesystem" ]; then
		grep -q "^$DEVNAME " /proc/mounts || automount
	fi
fi

if [ "$ACTION" = "remove" ] && [ -x "$UMOUNT" ] && [ -n "$DEVNAME" ] && [ -n "$ID_FS_USAGE" ]; then
	# ID_FS_USAGE has only been present for partitions
	# Another option would be DEVTYPE=partition? Other FS_USAGE are other, crypto
	if [ "$ID_FS_USAGE" = "filesystem" ]; then
		for mnt in `cat /proc/mounts | grep "$DEVNAME" | cut -f 2 -d " " `
		do
			$UMOUNT $mnt
			name="`basename "$mnt"`"
			test -e "/tmp/.automount-$name"
			rm_dir "$HOMED/$name"
		done
	fi
fi
